/***************************************************************************
 *   Copyright 2009  Petr Bravenec petr@bravenec.org                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License as        *
 *   published by the Free Software Foundation; either version 2 of        *
 *   the License or (at your option) version 3 or any later version        *
 *   accepted by the membership of KDE e.V. (or its successor approved     *
 *   by the membership of KDE e.V.), which shall act as a proxy            *
 *   defined in Section 14 of version 3 of the license.                    *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/

#include "openattachedfile.h"

// KDE includes
#include <kgenericfactory.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <kplugininfo.h>
#include <pluginsettings.h>
#include <kprocess.h>

// KMyMoney includes
#include <kmymoney/pluginloader.h>

typedef KGenericFactory<KMMOpenAttachedFilePlugin> openattachedfileFactory;

K_EXPORT_COMPONENT_FACTORY(kmm_openattachedfile, openattachedfileFactory("kmm_openattachedfile"))


/**
 * Konstruktor pluginu
 * Přidává akci do menu (podrobnosti jsou v kmm_openattachedfile.rc)
 * Propojuje vytvořenou akci v menu se slotem v pluginu
 */
KMMOpenAttachedFilePlugin::KMMOpenAttachedFilePlugin(QObject *parent, const QStringList&)
    : KMyMoneyPlugin::Plugin(parent, "Open Attached File" /* Musí být shodné s  X-KDE-PluginInfo-Name */) {

  setComponentData(openattachedfileFactory::componentData());
  setXMLFile("kmm_openattachedfile.rc");

  m_action = actionCollection()->addAction("openattachedfile", this, SLOT(slotOpenAttachedFile()));
  m_action->setText(i18n("Open Attached File"));
  m_action->setEnabled(false);

  connect(KMyMoneyPlugin::PluginLoader::instance(), SIGNAL(plug(KPluginInfo*)), this, SLOT(slotPlug(KPluginInfo*)));
  connect(KMyMoneyPlugin::PluginLoader::instance(), SIGNAL(unplug(KPluginInfo*)), this, SLOT(slotUnplug(KPluginInfo*)));
}


/**
 * Slot volaný po zasunutí.
 * Zajistí, aby se při změně vyběru transakcí zavolal slot v tomto pluginu
 */
void KMMOpenAttachedFilePlugin::slotPlug(KPluginInfo *info) {
  if (info->name() == objectName()) {
    connect(viewInterface(), SIGNAL(transactionsSelected(const KMyMoneyRegister::SelectedTransactions&)),
            this, SLOT(slotTransactionsSelected(const KMyMoneyRegister::SelectedTransactions&)));
    }
}


/**
 * Slot volaný po vysunutí.
 * Zajistí, aby se zrušila vazba mezi tímto pluginem a změnami ve výběru transakcí
 */
void KMMOpenAttachedFilePlugin::slotUnplug(KPluginInfo *info) {
  if (info->name() == objectName()) {
    disconnect(viewInterface(), SIGNAL(transactionsSelected(const KMyMoneyRegister::SelectedTransactions&)),
               this, SLOT(slotTransactionsSelected(const KMyMoneyRegister::SelectedTransactions&)));
  }

}


/**
 * Slot volaný při změně výběru transakcí.
 * Pokud vybrané transakce obsahují číslo dokumentu, povolí se akce spojená s transakcí, jinak
 * je akce v menu neaktivní.
 */
void KMMOpenAttachedFilePlugin::slotTransactionsSelected(const KMyMoneyRegister::SelectedTransactions& transactions) {
  bool actionEnabled = false;
  m_transactions = transactions;
  KMyMoneyRegister::SelectedTransactions::const_iterator it;
  for (it = transactions.constBegin(); it != transactions.constEnd(); it++) {
    if ( (*it).split().number() != "" ) {
      actionEnabled = true;
      break;
      }
    }
  m_action->setEnabled(actionEnabled);
}


/**
 * Tento slot je volaný při vyvolání akce z kontextového menu transakce.
 * Pro každou vybranou transakci se zadaným číslem dokladu
 * zavolá funkci pro otevření příslušného dokumentu.
 */
void KMMOpenAttachedFilePlugin::slotOpenAttachedFile() {
  qDebug() << "KMMOpenAttachedFilePlugin::slotOpenAttachedFile()";
  KMyMoneyRegister::SelectedTransactions::const_iterator it;
  for (it = m_transactions.constBegin(); it != m_transactions.constEnd(); ++it) {
    if ( (*it).split().number() == "" ) {
      continue;
      }
    openFile((*it).split().number());
    }
}


/**
 * Funkce pro otevření dokumentu v zadaném programu.
 * Program se startuje nezávisle na KMyMoney. Po ukončení
 * programu se volá slotProcessFinished, aby se po nastartovaném
 * a ukončeném programu uklidilo.
 */
void KMMOpenAttachedFilePlugin::openFile(const QString& number) {
  QStringList params = PluginSettings::path().split(' ');
  if (params.isEmpty()) return;
  KProcess *process = new KProcess();
  process->setProgram(params.takeFirst());

  for (int i=0; i<params.size(); i++) {
    *process << QString(params.at(i)).arg(number);
    }

  connect(process, SIGNAL(finished(int,QProcess::ExitStatus)),
          this,      SLOT(slotProcessFinished(int,QProcess::ExitStatus)));
  process->start();
}


/**
 * Slot je zavolaný po ukončení procesu s prohlížečem příslušného dokumentu.
 * Zajišťuje volání destruktoru třídy KProcess.
 */
void KMMOpenAttachedFilePlugin::slotProcessFinished(int,QProcess::ExitStatus) {
  sender()->deleteLater();
}

#include "openattachedfile.moc"
